#!/bin/bash
echo "** install mailserver **"
cd vmmail
# update os
apt-get -y update 
apt-get -y upgrade
# install dialog
apt-get -fym install dialog

# ask for domain, relayhost and pop3 server
INPUT="/tmp/input.txt"
dialog --title "Installation vmmail" --inputbox "Domain name" 10 40 2>$INPUT
response=$?
DOMAIN=$(cat $INPUT)
if [ "$response" = "1" ]
then 
  exit
fi
if [ "$response" = "255" ]
then 
  exit
fi

dialog --title "Installation vmmail" --inputbox "Relayhost (SMTP server)" 10 40 2>$INPUT
RELAYHOST=$(cat $INPUT)
response=$?
if [ "$response" = "1" ]
then 
  exit
fi
if [ "$response" = "255" ]
then 
  exit
fi

dialog --title "Installation vmmail" --inputbox "POP3 server" 10 40 2>$INPUT
POPHOST=$(cat $INPUT)
response=$?
if [ "$response" = "1" ]
then 
  exit
fi
if [ "$response" = "255" ]
then 
  exit
fi

dialog --title "Installation vmmail" --inputbox "Mail address SMTP" 10 40 2>$INPUT
MAILADDRESS=$(cat $INPUT)
response=$?
if [ "$response" = "1" ]
then 
  exit
fi
if [ "$response" = "255" ]
then 
  exit
fi

dialog --title "Installation vmmail" --inputbox "Mail password SMTP" 10 40 2>$INPUT
MAILPASSWORD=$(cat $INPUT)
response=$?
if [ "$response" = "1" ]
then 
  exit
fi
if [ "$response" = "255" ]
then 
  exit
fi

# packages dovecot roundcube
apt-get -fym install postfix dovecot-common dovecot-imapd mailutils mysql-server roundcube dovecot-managesieved dovecot-sieve fetchmail php-zip
# packages ssh, nfs
apt-get -fym install ssh sshpass nfs-client
# packages imapsync
#apt-get -fym install libio-tee-perl libmail-imapclient-perl libdigest-md5-file-perl libterm-readkey-perl libfile-copy-recursive-perl libunicode-string-perl libauthen-ntlm-perl libcrypt-ssleay-perl libdigest-hmac-perl libfile-copy-recursive-perl libio-compress-perl libio-socket-inet6-perl libio-socket-ssl-perl libio-tee-perl libmodule-scandeps-perl libnet-ssleay-perl libpar-packer-perl libreadonly-perl libterm-readkey-perl libtest-pod-perl libtest-simple-perl libunicode-string-perl liburi-perl libdate-manip-perl
# packages for stronger passwords
apt-get -fym install libpam-cracklib
# package for mkpasswd
apt-get -fym install whois
# package enigma/PGP
apt-get -fym install gnupg2
# packages wsgi flask
apt-get -fym install libapache2-mod-wsgi-py3 python3-pip python3-flask
# update to latest version dovecot
curl https://repo.dovecot.org/DOVECOT-REPO-GPG | gpg --import
gpg --export ED409DA1 > /etc/apt/trusted.gpg.d/dovecot.gpg
echo "deb https://repo.dovecot.org/ce-2.3-latest/ubuntu/focal focal main" > /etc/apt/sources.list.d/dovecot.list
apt update
apt -y upgrade

# backup mailserver system files
mkdir /backup
cp /etc/dovecot/conf.d/10-auth.conf /backup
cp /etc/dovecot/conf.d/10-mail.conf /backup
cp /etc/dovecot/conf.d/10-master.conf /backup
cp /etc/dovecot/conf.d/10-ssl.conf /backup
cp /etc/dovecot/conf.d/15-lda.conf /backup
cp /etc/dovecot/conf.d/20-managesieve.conf /backup
cp /etc/dovecot/conf.d/90-sieve.conf /backup
cp /etc/dovecot/conf.d/auth-system.conf.ext /backup
cp /etc/roundcube/defaults.inc.php /backup
cp /var/lib/roundcube/config/config.inc.php /backup
cp /etc/apache2/sites-available/default-ssl.conf /backup
cp /etc/apache2/sites-available/000-default.conf /backup
cp /etc/dovecot/dovecot.conf /backup
cp /etc/postfix/main.cf /backup
cp /etc/postfix/master.cf /backup
cp /etc/apache2/conf-available/roundcube.conf /backup
cp /etc/php/7.4/apache2/php.ini /backup
cp /etc/hostname /backup
cp /etc/hosts /backup
cp /etc/pam.d/common-password /backup

# system directories 
mkdir /mailserver
mkdir /mailserver/cert
mkdir /mailserver/util

# copy self created certificates
cp cert.pem /mailserver/cert
chmod 700 /mailserver/cert/cert.pem
cp key.pem /mailserver/cert
chmod 700 /mailserver/cert/key.pem
cp util/certificate.sh /mailserver/util
# user mailadmin
cp mailadmin.ini /mailserver
chown www-data:www-data /mailserver/mailadmin.ini

# install roundcube plugins
# =========================
# install vendor
tar xvfzp vendor.tar.gz
chown -R root:root vendor
mv vendor /usr/share/roundcube
ln -s /usr/share/roundcube/vendor /var/lib/roundcube

# install plugin ident_switch for roundcube
tar xvfzp ident_switch.tar.gz
cp -a ident_switch /usr/share/roundcube/plugins
chown -R root:root /usr/share/roundcube/plugins/ident_switch
ln -s /usr/share/roundcube/plugins/ident_switch /var/lib/roundcube/plugins
mysql roundcube < /usr/share/roundcube/plugins/ident_switch/SQL/mysql.initial.sql

# install plugin calendar for roundcube
tar xvfzp calendar.tar.gz
cp -a calendar/plugins/calendar /usr/share/roundcube/plugins
cp -a calendar/plugins/libcalendaring /usr/share/roundcube/plugins/
cp -a calendar/plugins/libkolab /usr/share/roundcube/plugins
cp /usr/share/roundcube/plugins/calendar/config.inc.php.dist /usr/share/roundcube/plugins/calendar/config.inc.php
# own file because of error "no dark mode available"
cp libkolab.min.css /usr/share/roundcube/plugins/libkolab/skins/elastic 
chown -R root:root /usr/share/roundcube/plugins/calendar
chown -R root:root /usr/share/roundcube/plugins/libcalendaring
chown -R root:root /usr/share/roundcube/plugins/libkolab
ln -s /usr/share/roundcube/plugins/calendar /var/lib/roundcube/plugins
ln -s /usr/share/roundcube/plugins/libcalendaring /var/lib/roundcube/plugins
ln -s /usr/share/roundcube/plugins/libkolab /var/lib/roundcube/plugins
# own mysql tables: change PRIMARY KEY in calendars to INDEX to make shared calendars
cp /usr/share/roundcube/plugins/calendar/drivers/database/SQL/mysql.initial.sql /usr/share/roundcube/plugins/calendar/drivers/database/SQL/mysql.initial.sql.orig
sed "s/PRIMARY KEY(\`calendar_id\`),/INDEX(\`calendar_id\`),/" -i /usr/share/roundcube/plugins/calendar/drivers/database/SQL/mysql.initial.sql
mysql roundcube < /usr/share/roundcube/plugins/calendar/drivers/database/SQL/mysql.initial.sql

# install plugin enigma/PGP
mkdir /enigma
chown -R www-data:www-data /enigma
ln -s /usr/share/roundcube/plugins/enigma /var/lib/roundcube/plugins

# install plugin synchronize address book with carddav
tar xvfzp carddav.tar.gz
cp -a carddav /usr/share/roundcube/plugins
cp /usr/share/roundcube/plugins/carddav/config.inc.php.dist /usr/share/roundcube/plugins/carddav/config.inc.php
chown -R root:root /usr/share/roundcube/plugins/carddav
ln -s /usr/share/roundcube/plugins/carddav /var/lib/roundcube/plugins

# update roundcube
ln -s /var/lib/roundcube/config /usr/share/roundcube
tar xfvzp roundcubemail-1.4.11-complete.tar.gz
cd roundcubemail-1.4.11
echo "y" | ./bin/installto.sh /usr/share/roundcube
cd ..
rm /usr/share/roundcube/config
rm /var/lib/roundcube/plugins/jqueryui/config.inc.php
ln -s /usr/share/roundcube/plugins/archive /var/lib/roundcube/plugins
ln -s /usr/share/roundcube/plugins/managesieve /var/lib/roundcube/plugins
ln -s /usr/share/roundcube/plugins/zipdownload /var/lib/roundcube/plugins
ln -s /usr/share/roundcube/plugins/password /var/lib/roundcube/plugins
ln -s /usr/share/roundcube/plugins/newmail_notifier /var/lib/roundcube/plugins
# change password with roundcube. use chpasswd
#cp password.php /usr/share/roundcube/plugins/password
cp /usr/share/roundcube/plugins/password/config.inc.php.dist /usr/share/roundcube/plugins/password/config.inc.php
sed -i "s/^\$config\['password_driver'\] = 'sql';/\$config['password_driver'] = 'chpasswd';/" /usr/share/roundcube/plugins/password/config.inc.php
# action after updat roundcube
# plugin enigma: config file completed with pgp home dir and binary
cp enigma.config.inc.php /usr/share/roundcube/plugins/enigma/config.inc.php

# change owner
chown -R root:root /usr/share/roundcube
echo "www-data ALL=(ALL) NOPASSWD: /usr/sbin/chpasswd" >> /etc/sudoers

# configuration fetchmail
cp fetchmail /etc/default
cp fetchmailrc /etc
chmod 1200 /etc/fetchmailrc
sed -i "s/POPHOST/$POPHOST/" /etc/fetchmailrc
cat /dev/null > /var/log/fetchmail.log
chown fetchmail /var/log/fetchmail.log
 
# create user template
maildirmake.dovecot /etc/skel/Maildir
maildirmake.dovecot /etc/skel/Maildir/.Drafts
maildirmake.dovecot /etc/skel/Maildir/.Sent
maildirmake.dovecot /etc/skel/Maildir/.Spam
maildirmake.dovecot /etc/skel/Maildir/.Trash
maildirmake.dovecot /etc/skel/Maildir/.Templates

# config dovecot
cp 10-auth.conf /etc/dovecot/conf.d/
cp 10-mail.conf /etc/dovecot/conf.d/
cp 10-master.conf /etc/dovecot/conf.d/
cp 10-ssl.conf /etc/dovecot/conf.d/
cp 15-lda.conf /etc/dovecot/conf.d/
cp 20-managesieve.conf /etc/dovecot/conf.d/
cp 90-sieve.conf /etc/dovecot/conf.d/
cp 17-replication.conf /etc/dovecot/conf.d/
cp config.inc.php /var/lib/roundcube/config/config.inc.php
cp dovecot.conf /etc/dovecot/dovecot.conf
# create user vmail for replication
adduser --disabled-password --disabled-login --no-create-home --gecos "" -uid 10000 vmail
echo vmail:Replicate2Mail. | chpasswd 
# allow /etc/passwd to replicate with user vmail
cat /etc/dovecot/conf.d/auth-system.conf.ext auth-system.conf.ext > /tmp/auth-system.conf.ext
mv /tmp/auth-system.conf.ext /etc/dovecot/conf.d/auth-system.conf.ext

# copy postfix system files
cp main.cf /etc/postfix/main.cf
cp master.cf /etc/postfix/master.cf
# config postfix
sed -i "s/DOMAIN/$DOMAIN/" /etc/postfix/main.cf
sed -i "s/RELAYHOST/$RELAYHOST/" /etc/postfix/main.cf
echo $RELAYHOST $MAILADDRESS:$MAILPASSWORD > /etc/postfix/sasl_passwd
postmap /etc/postfix/sasl_passwd
# copy apache system files
cp default-ssl.conf /etc/apache2/sites-available/
# extend ssl for mailadmin
sed -i "/<VirtualHost>/r 000-default-maildamin.conf" /etc/apache2/sites-available/000-default.conf
sed -i "s|DocumentRoot.*|DocumentRoot /var/www/mailadmin|g" /etc/apache2/sites-available/000-default.conf
#cp mailadmin.conf /etc/apache2/sites-available/
cp roundcube.conf /etc/apache2/conf-available/roundcube.conf

# create alias file for postfix
cp aliases /etc/aliases
newaliases

# attachment size 1 GB
sed -i "s/^\$config\['max_message_size'\].*$/\$config\['max_message_size'\] = '1000M';/" /etc/roundcube/defaults.inc.php
sed -i "s/^upload_max_filesize.*$/upload_max_filesize = 1000M/" /etc/php/7.4/apache2/php.ini
sed -i "s/^post_max_size.*$/post_max_size = 1000M/" /etc/php/7.4/apache2/php.ini
# display php errors
sed -i "s/^display_errors.*$/display_errors = On/" /etc/php/7.4/apache2/php.ini
# apache: rewrite http to https
echo "RewriteEngine On" >> /etc/roundcube/htaccess
echo "RewriteCond %{HTTPS} off" >> /etc/roundcube/htaccess
echo "RewriteRule (.*) https://%{HTTP_HOST}%{REQUEST_URI}" >> /etc/roundcube/htaccess

# applications to sync imap folders
#cp imapsync /usr/sbin
cp util/* /mailserver/util
# applications to config users and mailsevers
cp bin/* /usr/sbin

# exec scripts in www
echo "www-data ALL=(ALL) NOPASSWD: /usr/sbin/menable, /usr/sbin/madduser, /usr/sbin/mdisable, /usr/sbin/medituser, /usr/sbin/mpostfixrestart, /usr/sbin/mpostfixreload, /usr/sbin/msetdomain, /usr/sbin/msetfetchmailrc, /usr/sbin/msetpopserver, /usr/sbin/msetrelayhost, /usr/sbin/msetsmtp, /usr/sbin/mgetuser, /usr/sbin/mgetusers, /usr/sbin/msetpasswd, /usr/sbin/mcomppasswd, /usr/sbin/maddsync, /usr/sbin/meditsync, /usr/sbin/misuser, /usr/sbin/misenable, /usr/sbin/mlogapache, /usr/sbin/mlogmail, /usr/sbin/mlisthome, /usr/sbin/mlistmaildir, /usr/sbin/msharedadd, /usr/sbin/msharedinit, /usr/sbin/msharedlist, /usr/sbin/mshareduser, /usr/sbin/msharecalendar, /usr/bin/systemctl, /usr/sbin/mgetpop, /usr/sbin/mlogfetchmail, /usr/sbin/miscalendar, /usr/sbin/mlistcalendars" >> /etc/sudoers

# copy www
mkdir /var/www/mailadmin
cp -a www/* /var/www/mailadmin
cp appmailserver.wsgi /var/www/mailadmin
chown -R www-data:www-data /var/www/mailadmin

# server configuration
# ====================
# change host name
echo "mailserver" > /etc/hostname
# delete IPv6 in /etc/hosts
cp hosts /etc/hosts
# activate strongers passwords (length 10 characters, 1 upper case, 1 lower case, 1 special character, 1 integer)
sed -i "s/retry=.*//g" /etc/pam.d/common-password 
sed -i 's/pam_cracklib.so.$*/pam_cracklib.so retry=3 minlen=10 difok=3 ucredit=-1 lcredit=-1 dcredit=-1 ocredit=-1 enforce_for_root/g' /etc/pam.d/common-password
# log rotate for fetchmail.log
cp fetchmail.rotate /etc/logrotate.d/fetchmail
logrotate /etc/logrotate.conf
# Maildir for root
mkdir /root/Maildir
chmod a=rwx /root/Maildir

# restart mailserver services
systemctl restart fetchmail
systemctl restart postfix
systemctl restart dovecot

# enable SSL for apache
a2enmod ssl
a2ensite default-ssl.conf
# enable wsgi flask
a2enmod wsgi
#a2ensite mailadmin.conf
# enable http to https rewrite
a2enmod rewrite
# enable read and write file system
sed -i "s/PrivateTmp=.*/PrivateTmp=false/g" /lib/systemd/system/apache2.service
systemctl daemon-reload
systemctl restart apache2
echo ""
echo ""
echo "** reboot system **"
exit
